//----------------------------------------------------------------------
//							 Ks0108 Driver
//				   Write By Mohammad mazarei @ 1390/6/29
//					Email: mohammad.mazarei@gmail.com
//					 Publish In KavirElectronic.ir
//----------------------------------------------------------------------
#include <at91sam7x256.h>
#include <lib_at91sam7x256.h>
#include "ks0108.h"

#define KS0108_SDATA		(1<<ks0108_DATA)
#define ks0108_DATA_MASK	((KS0108_SDATA<<0)|(KS0108_SDATA<<1)|(KS0108_SDATA<<2)|(KS0108_SDATA<<3)|(KS0108_SDATA<<4)|(KS0108_SDATA<<5)|(KS0108_SDATA<<6)|(KS0108_SDATA<<7))
#define ks0108_datain		(char)((AT91F_PIO_GetInput(ks0108_PORT)>>ks0108_DATA)&0xFF)
#define ks0108_dataout(x)	AT91F_PIO_ClearOutput(ks0108_PORT,ks0108_DATA_MASK); AT91F_PIO_SetOutput(ks0108_PORT,((x & 0xFF)<<ks0108_DATA)) /*direction of the data pins and control pin*/
#define ks0108_dir_data_out	AT91F_PIO_CfgOutput(ks0108_PORT,ks0108_DATA_MASK)
#define ks0108_dir_data_in 	AT91F_PIO_CfgInput(ks0108_PORT,ks0108_DATA_MASK)

#define glcd_dir_control_out		AT91F_PIO_CfgOutput(ks0108_PORT,((1<<ks0108_RS)|(1<<ks0108_WR)|(1<<ks0108_E)|(1<<ks0108_CS0)|(1<<ks0108_CS1)|(1<<ks0108_CS2)|(1<<ks0108_CS3)|(1<<ks0108_RESET)))	

#define RWHI      AT91F_PIO_SetOutput(ks0108_PORT,(1<<ks0108_WR))
#define RWLO      AT91F_PIO_ClearOutput(ks0108_PORT,(1<<ks0108_WR))
#define RSHI      AT91F_PIO_SetOutput(ks0108_PORT,(1<<ks0108_RS))
#define RSLO      AT91F_PIO_ClearOutput(ks0108_PORT,(1<<ks0108_RS))
#define EHI       AT91F_PIO_SetOutput(ks0108_PORT,(1<<ks0108_E))
#define ELO       AT91F_PIO_ClearOutput(ks0108_PORT,(1<<ks0108_E))
#define CS0HI     AT91F_PIO_SetOutput(ks0108_PORT,(1<<ks0108_CS0))
#define CS0LO     AT91F_PIO_ClearOutput(ks0108_PORT,(1<<ks0108_CS0))
#define CS1HI     AT91F_PIO_SetOutput(ks0108_PORT,(1<<ks0108_CS1))
#define CS1LO     AT91F_PIO_ClearOutput(ks0108_PORT,(1<<ks0108_CS1))
#define CS2HI     AT91F_PIO_SetOutput(ks0108_PORT,(1<<ks0108_CS2))
#define CS2LO     AT91F_PIO_ClearOutput(ks0108_PORT,(1<<ks0108_CS2))
#define CS3HI     AT91F_PIO_SetOutput(ks0108_PORT,(1<<ks0108_CS3))
#define CS3LO     AT91F_PIO_ClearOutput(ks0108_PORT,(1<<ks0108_CS3))
#define RESETHI   AT91F_PIO_SetOutput(ks0108_PORT,(1<<ks0108_RESET))
#define RESETLO   AT91F_PIO_ClearOutput(ks0108_PORT,(1<<ks0108_RESET))



// absolute address of LCD Controller #0 CTRL and DATA registers
#define GLCD_CONTROLLER0_CTRL_ADDR    0x1000
#define GLCD_CONTROLLER0_DATA_ADDR    0x1001
// offset of other controllers with respect to controller0
#define GLCD_CONTROLLER_ADDR_OFFSET    0x0002





// HD61202/KS0108 command set
#define GLCD_ON_CTRL        0x3E    // 0011111X: lcd on/off control
#define GLCD_ON_DISPLAY        0x01    //        DB0: turn display on

#define GLCD_START_LINE        0xC0    // 11XXXXXX: set lcd start line

#define GLCD_SET_PAGE        0xB8    // 10111XXX: set lcd page (X) address
#define GLCD_SET_Y_ADDR        0x40    // 01YYYYYY: set lcd Y address

#define GLCD_STATUS_BUSY     0x80    // (1)->LCD IS BUSY
#define GLCD_STATUS_ONOFF    0x20    // (0)->LCD IS ON
#define GLCD_STATUS_RESET    0x10    // (1)->LCD IS RESET

// determine the number of controllers
// (make sure we round up for partial use of more than one controller)
#define GLCD_NUM_CONTROLLERS    ((GLCD_XPIXELS+GLCD_CONTROLLER_XPIXELS-1)/GLCD_CONTROLLER_XPIXELS)



typedef struct struct_GrLcdCtrlrStateType
{
	unsigned char xAddr;
	unsigned char yAddr;
} GrLcdCtrlrStateType;

typedef struct struct_GrLcdStateType
{
	unsigned char lcdXAddr;
	unsigned char lcdYAddr;
	GrLcdCtrlrStateType ctrlr[GLCD_NUM_CONTROLLERS];
} GrLcdStateType;



static GrLcdStateType GrLcdState;




void ks0108_InitHW(void)
{

    //TODO: make setup of chip select lines contingent on how
    // many controllers are actually in the display
	AT91C_BASE_PMC->PMC_PCER = ( 1<<ks0108_PMC );
    //AT91F_PIO_CfgPullup(ks0108_PORT,ks0108_DATA_MASK);
	 // initialize LCD control lines levels
    RSLO;
    RWLO;
    ELO;
    CS0LO;
    CS1LO;
    CS2LO;
    CS3LO;
    RESETLO;
    // initialize LCD control port to output
    glcd_dir_control_out;
    // initialize LCD data
    ks0108_dataout(0x00);
    // initialize LCD data port to output
	ks0108_dir_data_out;
	ks0108_dataout(0xFF);
	
}

void ks0108_ControllerSelect(unsigned char controller)
{

    //TODO: make control of chip select lines contingent on how
    // many controllers are actually in the display

    // unselect all controllers
    //CS0LO;
    //CS1LO;
    //CS2LO;
    //CS3LO;
	AT91F_PIO_ClearOutput(ks0108_PORT,((1<<ks0108_CS0)|(1<<ks0108_CS1)|(1<<ks0108_CS2)|(1<<ks0108_CS3)));

    // select requested controller
    switch(controller)
    {
    case 0:    CS0HI; break;
    case 1:    CS1HI; break;
	case 2:	   CS2HI; break;
	case 3:	   CS3HI; break;
	default: break;
	}

}

void ks0108_BusyWait(unsigned char controller)
{	

	char temp;
	// wait until LCD busy bit goes to zero
	// select the controller chip
	ks0108_ControllerSelect(controller);
	// do a read from control register
	//ks0108_dataout(0xFF);
	ks0108_dir_data_in;
	RSLO;
	RWHI;
	EHI;
	DELAY;
	
	temp = 	ks0108_datain ;
	while( temp & GLCD_STATUS_BUSY)
	{
		
		ELO;
		DELAY;
		EHI;
		DELAY;
		
		temp = 	ks0108_datain ;
		
	}
	
	ELO;
	RWLO;
	ks0108_dir_data_out;
  

}

void ks0108_ControlWrite(unsigned char controller, unsigned char data)
{


	ks0108_BusyWait(controller);	// wait until LCD not busy
	RSLO;
	RWLO;
	EHI;
	//ks0108_dir_data_out;
	ks0108_dataout(data);
	DELAY;
	ELO;

}

void ks0108_SetYAddress(unsigned char yAddr)
{
	unsigned char i;
	// record address change locally
	GrLcdState.lcdYAddr = yAddr;
	// set page address for all controllers
	for(i=0; i<GLCD_NUM_CONTROLLERS; i++)
	{
		ks0108_ControlWrite(i, GLCD_SET_PAGE | yAddr);
	}
}

unsigned char ks0108_ControlRead(unsigned char controller)
{
	register unsigned char data;


	ks0108_BusyWait(controller);		// wait until LCD not busy
	RSLO;
	ks0108_dir_data_in;
	RWHI;
	EHI;
	DELAY;
	data = ks0108_datain;
	ELO;
	RWLO;
	ks0108_dir_data_out;
	return data;
}

void ks0108_Write(unsigned char data)
{
	register unsigned char controller = (GrLcdState.lcdXAddr/GLCD_CONTROLLER_XPIXELS);


	ks0108_BusyWait(controller);		// wait until LCD not busy
	RSHI;
	RWLO;
	EHI;
	//ks0108_dir_data_out;
	ks0108_dataout(data);
	DELAY;
	ELO;


	// increment our local address counter
	GrLcdState.ctrlr[controller].xAddr++;
	GrLcdState.lcdXAddr++;
	if(GrLcdState.lcdXAddr >= GLCD_XPIXELS)
	{
		GrLcdState.lcdYAddr++;
		ks0108_SetYAddress(GrLcdState.lcdYAddr);
		ks0108_SetXAddress(0);
	}
}

void ks0108_Writexy(unsigned char x,unsigned char y,unsigned char data){
 ks0108_gotoxy(x,y);
 ks0108_Write(data);
}

void ks0108_SetXAddress(unsigned char xAddr)
{
	unsigned char i;
	// record address change locally
	GrLcdState.lcdXAddr = xAddr;

	// clear y (col) address on all controllers
	for(i=0; i<GLCD_NUM_CONTROLLERS; i++)
	{
		ks0108_ControlWrite(i, GLCD_SET_Y_ADDR | 0x00);
		GrLcdState.ctrlr[i].xAddr = 0;
	}

	// set y (col) address on destination controller
	ks0108_ControlWrite((GrLcdState.lcdXAddr/GLCD_CONTROLLER_XPIXELS),
		GLCD_SET_Y_ADDR | (GrLcdState.lcdXAddr & 0x3F));
}



unsigned char ks0108_Read(void)
{
	register unsigned char data;
	register unsigned char controller = (GrLcdState.lcdXAddr/GLCD_CONTROLLER_XPIXELS);


	ks0108_BusyWait(controller);		// wait until LCD not busy
	RSHI;
	ks0108_dir_data_in;
	RWHI;
	EHI;
	DELAY;
	data = ks0108_datain;
	ELO;
	RWLO;
	//DUMMY BYTE
    
	ks0108_BusyWait(controller);		// wait until LCD not busy
	RSHI;
	ks0108_dir_data_in;
	RWHI;
	EHI;
	DELAY;
	

	data = ks0108_datain;
	ELO;
	RWLO;


	// increment our local address counter
	GrLcdState.ctrlr[controller].xAddr++;
	GrLcdState.lcdXAddr++;
	if(GrLcdState.lcdXAddr >= GLCD_XPIXELS)
	{
		GrLcdState.lcdYAddr++;
		ks0108_SetYAddress(GrLcdState.lcdYAddr);
		ks0108_SetXAddress(0);
	}
	return data;
}

unsigned char  ks0108_Readxy(unsigned char x,unsigned char y){
 ks0108_gotoxy(x,y);
 return ks0108_Read();
}

void ks0108_Reset(unsigned char resetState)
{
	// ks0108_Reset lcd if argument is true
	// run lcd if argument is false

	if(resetState){
		RESETLO;
	}else{
		RESETHI;}

}

/*************************************************************/
/********************* PUBLIC FUNCTIONS **********************/
/*************************************************************/

void ks0108_Init(void)
{
	unsigned char i;
    
	// initialize hardware
	ks0108_InitHW();
    
	// bring lcd out of ks0108_Reset
	ks0108_Reset(FALSE);
	// Turn on LCD
	for(i=0; i<GLCD_NUM_CONTROLLERS; i++)
	{
		ks0108_ControlWrite(i, GLCD_ON_CTRL | GLCD_ON_DISPLAY);
	}
	// clear lcd
	ks0108_ClearScreen();
	// initialize positions
	ks0108_Home();
}


void ks0108_Home(void)
{
	unsigned char i;
	// initialize addresses/positions
	ks0108_StartLine(0);
	ks0108_SetAddress(0,0);
	// initialize local data structures
	for(i=0; i<GLCD_NUM_CONTROLLERS; i++)
	{
		GrLcdState.ctrlr[i].xAddr = 0;
		GrLcdState.ctrlr[i].yAddr = 0;
	}
}

void ks0108_ClearScreen(void)
{
	unsigned char pageAddr;
	unsigned char xAddr;

	// clear LCD
	// loop through all pages
	for(pageAddr=0; pageAddr<(GLCD_YPIXELS>>3); pageAddr++)
	{
		// set page address
		ks0108_SetAddress(0, pageAddr);
		// clear all lines of this page of display memory
		for(xAddr=0; xAddr<GLCD_XPIXELS; xAddr++)
		{
			ks0108_Write(0x00);
		}
	}
}

void ks0108_StartLine(unsigned char start)
{
	ks0108_ControlWrite(0, GLCD_START_LINE | start);
	ks0108_ControlWrite(1, GLCD_START_LINE | start);
}

void ks0108_SetAddress(unsigned char x, unsigned char yLine)
{
	// set addresses
	ks0108_SetYAddress(yLine);
	ks0108_SetXAddress(x);
}
void ks0108_gotoxy(unsigned char x, unsigned char yLine){
  	ks0108_SetYAddress(yLine);
	ks0108_SetXAddress(x);
}

 void ks0108_pset(unsigned char x,unsigned char y)
{
	unsigned char temp;
	   if ((x>= GLCD_XPIXELS )||(y>= GLCD_YPIXELS)) return;
	temp = ks0108_Readxy(x,y/8); // ks0108_Read Byte in LCD RAM
	ks0108_Writexy(x, y/8,temp | (1 << (y % 8)));
}


void ks0108_ResetPoint(unsigned char x,unsigned char y)
{
	unsigned char temp;
	temp = ks0108_Readxy(x,y/8); // ks0108_Read Byte in LCD RAM
	ks0108_Writexy(x,y/8,temp & ~(1 << (y % 8)));

}

void ks0108_Point(unsigned char x,unsigned char y,unsigned char _point){

	unsigned char temp;
	if ((x>= GLCD_XPIXELS )||(y>= GLCD_YPIXELS)) return;
	temp = ks0108_Readxy(x,y/8); // ks0108_Read Byte in LCD RAM
	if(_point==0)
        ks0108_Writexy(x,y/8,temp & ~(1 << (y % 8)));
	else
        ks0108_Writexy(x,y/8,temp | (1 << (y % 8)));

}

void ks0108_show_pic(char *pic,char x, char y){

char  i,j,XS,YS;
       XS=*pic;
       YS=*(pic+1);
    for(j=0;j<YS;j++){
        ks0108_gotoxy(0+x,j+y);;   
    for(i=0;i<XS;i++)
        {
        ks0108_Write(pic[j*XS+i+2]);   
            }
  }
}

void ks0108_show_picf(const char *pic,char x, char y){

char  i,j,XS,YS;
       XS=*pic;
       YS=*(pic+1);
    for(j=0;j<YS;j++){
        ks0108_gotoxy(0+x,j+y);;   
    for(i=0;i<XS;i++)
        {
        ks0108_Write(pic[j*XS+i+2]);   
            }
  }
}


